#
# engine.py - handles generally running the game
# can set a game resolution (iwidth, iheight) as well as a screen resolution
# the screen scales to fit
# also has a builting framerate counter
#

import os
import pygame
import pygame.mixer
from core import world
import agents

def fit(surf,size):
    surf = pygame.transform.scale(surf,size)
    return surf

class GameChoiceWorld(world.World):
    def start(self):
        x = 20
        y = 20
        self.add(agents.TextAgent(["Pick game:"],[0,0]))
        for game in self.engine.games:
            t = agents.TextAgent([game],[x,y])
            t.game = game
            self.add(t)
            y+=10
    def input(self,controller):
        if controller.mouse_clicked:
            pos = list(controller.mouse_clicked.pos)
            pos[0]=int(pos[0]*0.5)
            pos[1]=int(pos[1]*0.5)
            for o in self.objects:
                if not hasattr(o,"game"):
                    continue
                if o.rect().collidepoint(pos):
                    self.engine.choose_game_type(o.game)
                    self.engine.make_world()

class Engine:
    def __init__(self):
        self.game_name = "No game loaded"
        self.game_module = None
        self.readme_mode = False
        self.fullscreen = False
        #The screen width, what resolution the screen is scaled to
        self.swidth = 640
        self.sheight = 480
        #The interactive width, what resolution the game is actually rendered at
        self.iwidth = 320
        self.iheight = 240
        self.window = None   #The window is the actual window
        self.surface = None   #The surface is what will be displayed, most of the time draw to this
        self.blank = None
        self.running = False   #If this is set to false, the game will quit
        self.paused = False   #Not implemented, should be controlled by the world
        self.framerate = 60    #What framerate the game runs at
        self.dt = 0
        self.show_fps = False
        self.clock = None
        self.world = None   #Change what world is set to to change between scenes or modes
        self.next_tick = 0.0
        self.sounds = {}
    def start(self):
        """Separate from __init__ in case we want to make the object before making the screen"""
        pygame.init()
        self.make_screen()
        self.running = True
        self.clock = pygame.time.Clock()
        self.font = pygame.font.Font("fonts/vera.ttf",8)
        pygame.mixer.init()
    def sound(self,s):
        if s not in self.sounds:
            self.sounds[s] = pygame.mixer.Sound(s)
        self.sounds[s].play()
    def load_game_type(self,name=None):
        #If no name given, show menu
        games = []
        for g in os.listdir("core"):
            f = "core/"+g
            if os.path.isdir(f) and os.path.exists(f+"/readme.txt"):
                if not os.path.exists(f+"/__init__.py"):
                    open(f+"/__init__.py","w").close()
                if name is None or g==name:
                    games.append(g)
        if len(games)==1:
            g = games[0]
            self.choose_game_type(games[0])
        else:
            self.games = games
    def choose_game_type(self,g):
        f = "core/"+g
        self.game_path = f
        self.game_short_name = g
        self.game_module = __import__(g+".blah",globals(),locals())
        self.game_name = open(f+"/readme.txt").read().split("\n")[0]
        self.game_readme = open(self.game_path+"/readme.txt").read().split("\n")
        self.make_screen()
    def make_world(self):
        if self.game_module:
            self.world = self.game_module.blah.make_world(self)
            self.show_readme()
        else:
            self.world = GameChoiceWorld(self)
    def show_readme(self):
        if self.readme_mode:
            return
        self.readme_mode = True
        self.last_world = self.world
        self.world = world.ReadmeWorld(self,self.game_readme)
    def hide_readme(self):
        if not self.readme_mode:
            return
        self.readme_mode = False
        self.world = self.last_world
    def stop(self):
        self.running = False
    def pause(self):
        self.paused = True
    def unpause(self):
        self.paused = False
    def update(self):
        """One tick, according to dt"""
        self.next_tick += self.dt
        if self.world:
            while self.next_tick>0:
                self.next_tick -= 1
                self.world.update()
    def make_screen(self):
        flags = pygame.RESIZABLE|pygame.FULLSCREEN*self.fullscreen
        self.window = pygame.display.set_mode([self.swidth,self.sheight],flags)
        self.surface = pygame.Surface([self.iwidth,self.iheight]).convert()
        self.blank = self.surface.convert()
        self.blank.fill([0,0,0])
        pygame.display.set_icon(pygame.image.load("art/icons/ico.png"))
        pygame.display.set_caption(self.game_name)
    def screenshot(self,name=None):
        if not name:
            name = self.game_short_name+"_screen.jpg"
        pygame.image.save(self.window,self.game_path+"/"+name)
    def clear_screen(self):
        self.surface.blit(self.blank,[0,0])
    def draw_screen(self):
        self.window.fill([10,10,10])
        def draw_segment(dest,surf,pos,size):
            rp = [pos[0]*self.swidth,pos[1]*self.sheight]
            rs = [size[0]*self.swidth,size[1]*self.sheight]
            surf = fit(surf,rs)
            dest.blit(surf,rp)
        draw_segment(self.window,self.surface,[0,0],[1,1])
        if self.show_fps:
            self.window.blit(self.font.render(str(self.clock.get_fps()),1,[255,0,0]),[0,self.window.get_height()-12])
        pygame.display.flip()